package com.caimei.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.caimei.model.ResponseJson;
import com.caimei.model.dto.ShopSaveDto;
import com.caimei.model.vo.ShopVo;
import com.caimei.service.ShopService;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.Map;

/**
 * 供应商API
 *
 * @author : Aslee
 * @date : 2021/5/11
 */
@Api(tags = "供应商API")
@Slf4j
@RestController
@RequiredArgsConstructor
@RequestMapping("/shop")
public class ShopApi {

    private final ShopService shopService;

    /**
     * 供应商列表
     */
    @ApiOperation("供应商列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "shopName", required = false, value = "供应商名称"),
            @ApiImplicitParam(name = "shopType", required = false, value = "供应商类型：1品牌方，2代理商"),
            @ApiImplicitParam(name = "brandId", required = false, value = "所属品牌id"),
            @ApiImplicitParam(name = "mobile", required = false, value = "手机号"),
            @ApiImplicitParam(name = "linkMan", required = false, value = "联系人"),
            @ApiImplicitParam(name = "pageNum", required = false, value = "第几页"),
            @ApiImplicitParam(name = "pageSize", required = false, value = "一页多少条")
    })
    @GetMapping("/list")
    public ResponseJson<PageInfo<ShopVo>> getShopList(String shopName, Integer shopType, Integer brandId, String mobile, String linkMan,
                                                      @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                                      @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) {

        return shopService.getShopList(shopName, shopType, brandId, mobile, linkMan, pageNum, pageSize);
    }

    /**
     * 获取供应商表单数据
     * @param authUserId    供应商用户id
     * @return  ShopVo
     */
    @GetMapping("/form/data")
    public ResponseJson<ShopVo> getShopFormData(Integer authUserId) {
        return shopService.getShopFormData(authUserId);
    }

    /**
     * 添加供应商
     * @param shopSaveDto {
     *                   shopType           供应商类型：1代理商，2品牌方
     *                   brandId            品牌id
     *                   shopName           供应商名称
     *                   mobile             手机号
     *                   linkMan            联系人
     *                   countryId          产地国家id
     *                   brandAuthLogo      品牌授权logo
     *                   shopStatus         供应商状态：0停用 1启用
     *                   securityLink       官网认证链接
     *                   statementType      代理声明类型：1弹窗 2链接 3图片 4文件（.doc .ppt .pdf）
     *                   statementContent   声明弹窗内容
     *                   statementLink      声明链接
     *                   statementImage     声明图片
     *                   statementFileId    声明文件id
     *                   createBy           创建人用户id
     * }
     */
    @ApiOperation("添加/编辑供应商")
    @PostMapping("/save")
    public ResponseJson saveShop(@RequestBody ShopSaveDto shopSaveDto) {
        return shopService.saveShop(shopSaveDto);
    }

    /**
     * 更新供应商状态
     */
    @ApiOperation("更新供应商状态")
    @ApiImplicitParam(name = "params", value = "authUserId:供应商用户id;status:供应商状态:0停用 1启用", required = true)
    @PostMapping("/update/status")
    public ResponseJson updateShopStatus(@RequestBody Map<String,Integer> params) {
        Integer authUserId = params.get("authUserId");
        Integer status = params.get("status");
        return shopService.updateShopStatus(authUserId, status);
    }

    /**
     * 重置密码
     */
    @ApiOperation("重置密码")
    @ApiImplicitParam(name = "params", value = "authUserId:供应商用户id", required = true)
    @PostMapping("/reset/password")
    public ResponseJson resetShopPassword(@RequestBody Map<String,Integer> params) {
        Integer authUserId = params.get("authUserId");
        return shopService.resetShopPassword(authUserId);
    }


    /**
     * 代理声明文件上传
     *
     * @param params authUserId:供应商用户id,file:上传的文件,fileName:文件名称
     * @return
     */
    @ApiOperation("代理声明文件上传")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "authUserId", required = false, value = "供应商用户id"),
            @ApiImplicitParam(name = "file", required = false, value = "代理声明文件"),
    })
    @PostMapping("/upload/file")
    public ResponseJson<Integer> uploadFile(Integer authUserId, MultipartFile file) {
        /*if (StringUtils.isBlank(params)) {
            return ResponseJson.error("数据异常", null);
        }
        Integer authUserId = null;
        MultipartFile file = null;
        try {
            JSONObject paramsMap = JSON.parseObject(params);
            if (null == paramsMap) {
                return ResponseJson.error("数据异常，参数不能为空", null);
            }
            authUserId = paramsMap.getInteger("authUserId");
            file = (MultipartFile) paramsMap.get("file");
            if (null == file) {
                return ResponseJson.error("数据异常，上传文件不能为空", null);
            }
        } catch (Exception e) {
            log.error("<<<<< 系统异常 >>>>>");
        }*/
        return shopService.uploadFile(authUserId, file);
    }
}
